﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using Windows.ApplicationModel.Resources.Core;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using System.Collections.Specialized;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using FlexGridSamples;

// The data model defined by this file serves as a representative example of a strongly-typed
// model that supports notification when members are added, removed, or modified.  The property
// names chosen coincide with data bindings in the standard item templates.
//
// Applications may use this model as a starting point and build on it, or discard it entirely and
// replace it with something appropriate to their needs.

namespace FlexGridSamples.Data
{
    /// <summary>
    /// Base class for <see cref="SampleDataItem"/> that defines common properties.
    /// </summary>
    [Windows.Foundation.Metadata.WebHostHidden]
    public abstract class SampleDataCommon : FlexGridSamples.Common.BindableBase
    {
        private static Uri _baseUri = new Uri("ms-appx:///");

        public SampleDataCommon(String uniqueId, String title, String description, String imagePath, String name)
        {
            this._uniqueId = uniqueId;
            this._title = title;
            this._description = description;
            this._imagePath = imagePath;
            this.Name = name;
        }

        private string _uniqueId = string.Empty;
        public string UniqueId
        {
            get { return this._uniqueId; }
            set { this.SetProperty(ref this._uniqueId, value); }
        }

        private string _title = string.Empty;
        public string Title
        {
            get { return this._title; }
            set { this.SetProperty(ref this._title, value); }
        }

        private string _description = string.Empty;
        public string Description
        {
            get { return this._description; }
            set { this.SetProperty(ref this._description, value); }
        }

        public string Name { get; set; }

        public DataTemplate Icon
        {
            get
            {
                try
                {
                    return (DataTemplate)App.Current.Resources["IconC1" + UniqueId.Replace(" ", "")];
                }
                catch
                {
                    return (DataTemplate)App.Current.Resources["IconC1Gray"];
                }
            }
        }

        private ImageSource _image = null;
        private String _imagePath = null;
        public ImageSource Image
        {
            get
            {
                if (this._image == null && this._imagePath != null)
                {
                    this._image = new BitmapImage(new Uri(SampleDataCommon._baseUri, this._imagePath));
                }
                return this._image;
            }

            set
            {
                this._imagePath = null;
                this.SetProperty(ref this._image, value);
            }
        }

        public void SetImage(String path)
        {
            this._image = null;
            this._imagePath = path;
            this.OnPropertyChanged("Image");
        }

        public override string ToString()
        {
            return this.Title;
        }
    }

    /// <summary>
    /// Generic item data model.
    /// </summary>
    public class SampleDataItem : SampleDataCommon
    {
        public SampleDataItem(String uniqueId, String title, String description, String imagePath, Type pageType, string name)
            : base(uniqueId, title, description, imagePath, name)
        {
            _pageType = pageType;
        }

        private Type _pageType;
        public Type PageType
        {
            get { return this._pageType; }
            set { this.SetProperty(ref this._pageType, value); }
        }
    }

    /// <summary>
    /// Creates a collection of groups and items with hard-coded content.
    /// 
    /// SampleDataSource initializes with placeholder data rather than live production
    /// data so that sample data is provided at both design-time and run-time.
    /// </summary>
    public sealed class SampleDataSource
    {
        private static SampleDataSource _sampleDataSource = new SampleDataSource();

        private ObservableCollection<SampleDataItem> _allItems = new ObservableCollection<SampleDataItem>();
        public ObservableCollection<SampleDataItem> AllItems
        {
            get { return this._allItems; }
        }

        public static IEnumerable<SampleDataItem> GetItems(string uniqueId)
        {
            if (!uniqueId.Equals("AllItems")) throw new ArgumentException(Strings.UniqueIdItemsArgumentException);

            return _sampleDataSource.AllItems;
        }

        public static SampleDataItem GetItem(string uniqueId)
        {
            // Simple linear search is acceptable for small data sets
            var matches = _sampleDataSource.AllItems.Where((item) => item.UniqueId.Equals(uniqueId));
            if (matches.Count() == 1) return matches.First();
            return null;
        }

        public SampleDataSource()
        {
             _allItems.Add(new SampleDataItem("Financial",
                 Strings.FinancialTitle,
                 Strings.FinancialDescription,
                 "Assets/Financial.png",
                 typeof(Financial),
                 Strings.FinancialName));
             _allItems.Add(new SampleDataItem("CollectionView",
                 Strings.CollectionViewTitle,
                 Strings.CollectionViewDescription,
                 "Assets/CollectionView.png",
                 typeof(CollectionView),
                 Strings.CollectionViewName));
             _allItems.Add(new SampleDataItem("Media Library",
                 Strings.MediaLibraryTitle,
                 Strings.MediaLibraryDescription,
                 "Assets/itunes.png",
                 typeof(MediaLibrary),
                 Strings.MediaLibraryName));
             _allItems.Add(new SampleDataItem("Editing",
                 Strings.EditingTitle,
                 Strings.EditingDescription,
                 "Assets/Editing.png",
                 typeof(Editing),
                 Strings.EditingName));
             _allItems.Add(new SampleDataItem("Unbound",
                 Strings.UnboundTitle,
                 Strings.UnboundDescription, 
                 "Assets/Unbound.png",
                 typeof(Unbound),
                 Strings.UnboundName));
             _allItems.Add(new SampleDataItem("CustomColumns",
                 Strings.CustomColumnsTitle,
                 Strings.CustomColumnsDescription,
                 "Assets/CustomColumns.png",
                 typeof(CustomColumns),
                 Strings.CustomColumnsName));
             _allItems.Add(new SampleDataItem("Cell Merging",
                 Strings.CellMergingTitle,
                 Strings.CellMergingDescription, 
                 "Assets/CellMerging.png",
                 typeof(ExcelStyleMerge),
                 Strings.CellMergingName));
             _allItems.Add(new SampleDataItem("Printing",
                 Strings.PrintingTitle,
                 Strings.PrintingDescription, 
                 "Assets/Printing.png",
                 typeof(Printing),
                 Strings.PrintingName));
            //_allItems.Add(new SampleDataItem("YouTube",
            //     Strings.YouTubeName,
            //     Strings.YouTubeTitle,
            //     "Assets/YouTube.png",
            //     typeof(YouTube),
            //     Strings.YouTubeDescription));
            _allItems.Add(new SampleDataItem("Drag-drop",
                 Strings.DragDropTitle,
                 Strings.DragDropDescription, 
                 "Assets/DragDropRows.png",
                 typeof(DragDropRows),
                 Strings.DragDropName));
            _allItems.Add(new SampleDataItem("RowDetails",
                 Strings.RowDetailsTitle,
                 Strings.RowDetailsDescription,
                 "Assets/RowDetails.png",
                 typeof(RowDetails),
                 Strings.RowDetailsName));
        }
    }
}
